@@ -31,6 +31,7 @@ __all__ = ['Configuration',
            'TypeCastException',
            'ConfigurationOption']
 
+
 ### exceptions
 
 class UnknownOptionException(Exception):
@@ -65,15 +66,11 @@ class ConfigurationProvider(object):
         else:
             f = filename
             newfile = False
-        exception = None
         try:
             self._write(f, config)
-        except Exception, exception:
-            pass
-        if newfile:
-            f.close()
-        if exception:
-            raise exception
+        finally:
+            if newfile:
+                f.close()
     def _write(self, fp, config):
         raise NotImplementedError("Abstract base class")
 
@@ -85,7 +82,6 @@ if json:
             return json.loads(file(filename).read())
         def _write(self, fp, config):
             fp.write(json.dumps(config, indent=self.indent, sort_keys=True))
-            # TODO: could use templates to get order down, etc
     configuration_providers.append(JSON())
 
 if yaml:
@@ -109,6 +105,7 @@ if yaml:
 
 __all__.extend([i.__class__.__name__ for i in configuration_providers])
 
+
 ### optparse interface
 
 class ConfigurationOption(optparse.Option):
@@ -131,6 +128,7 @@ class ConfigurationOption(optparse.Option):
             parser.parsed = dict()
         parser.parsed[dest] = value
 
+
 ### plugins for option types
 
 class BaseCLI(object):
@@ -156,6 +154,7 @@ class BaseCLI(object):
     def take_action(self, value):
         return value
 
+
 class BoolCLI(BaseCLI):
 
     def __call__(self, name, value):
@@ -240,6 +239,13 @@ class Configuration(optparse.OptionParser):
     load_option = 'load' # where to put the load option
     load_help = "load configuration from a file"
 
+    @classmethod
+    def parse(cls, args, *_args, **_kwargs):
+        """get the resultant config dictionary in a single call"""
+        conf = cls(*_args, **_kwargs)
+        conf.parse_args(*_args, **_kwargs)
+        return conf.config
+
     def __init__(self, configuration_providers=configuration_providers, types=types, load=None, dump='--dump', **parser_args):
 
         # sanity check
@@ -358,6 +364,12 @@ class Configuration(optparse.OptionParser):
                 defaults[key] = value['default']
         return copy.deepcopy(defaults)
 
+    def get(self, key, default=None):
+        return self.config.get(key, default)
+
+    def __getitem__(self, key):
+        return self.config[key]
+
     def __call__(self, *args):
         """add items to configuration and check it"""
         # TODO: configuration should be locked after this is called
@@ -579,19 +591,19 @@ class UserConfiguration(Configuration):
     """`Configuration` class driven by a config file in user-space"""
 
     # configuration items to interpolate as paths
-    # TODO: moar bettur!
+    # TODO: integrate with above BaseCLI
     paths = []
 
-    def __init__(self, config=None, load='--config'):
+    def __init__(self, config=None, load='--config', **kwargs):
 
         # default configuration file
         self.config_name = config or '.' + os.path.splitext(sys.argv[0])[0]
         self.default_config_file = os.path.join('~', self.config_name)
         self.default_config_file_path = os.path.expanduser(self.default_config_file)
         if os.path.exists(self.default_config_file_path):
-            self.load_help += ' [DEFAULT: %s]' % default_config_file
+            self.load_help += ' [DEFAULT: %s]' % self.default_config_file
 
-        Configuration.__init__(self, usage=self.usage, load=load)
+        Configuration.__init__(self, load=load, **kwargs)
 
     def validate(self):
         Configuration.validate(self)
@@ -603,7 +615,7 @@ class UserConfiguration(Configuration):
         if not configuration_files:
             # load user config only if no config provided
             if os.path.exists(self.default_config_file_path):
-                configuration_files = [default_config]
+                configuration_files = [self.default_config_file_path]
         return configuration_files
 
     def load_configuration_file(self, filename):
